/******************************************************************************
  permmat2maxmin.c                                           2011/05/25 tesakai

A program for randomization test

INPUT: a topic-system permutation matrix

MNUM: number of systems (#columns)
NNUM: number of topics (#rows in original matrix)
BNUM: number of trials

e.g.
<bnum> <system1> <system2> <system3>
1 0.100000 0.200000 0.300000 
1 0.900000 0.900000 0.800000 
1 0.100000 0.200000 0.500000 
1 0.000000 0.900000 0.100000 
2 0.200000 0.300000 0.100000 
2 0.900000 0.900000 0.800000 
2 0.100000 0.500000 0.200000 
2 0.100000 0.000000 0.900000 
3 0.300000 0.100000 0.200000 
3 0.800000 0.900000 0.900000 
3 0.500000 0.100000 0.200000 
3 0.900000 0.000000 0.100000


OUTPUT:

<bnum> <maxsum> <minsum> <maxmean-minmean>
1 2.2 1.1 0.275
2 2 1.3 0.175
3 2.5 1.1 0.35

where
- maxsum is the max sum across system columns for each trial (max perf system);
- minsum is the min sum across system columns for each trial (min perf system);
- maxmean-minmean is the mean diff between these two systems.

******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "strmanip.h"

#define SYSNUMMAX 1000
#define SEP " "

main( ac, av )
 int ac;
 char **av;
{
  FILE *f;
  char *buf;
  char *p;
  
  long sysnum;
  long topicnum;
  long trialnum;
  long linecount;
  long trialcount;
  long i;

  double sum[ SYSNUMMAX ];
  double max, min;

  if( ac < 5 ){

    fprintf( stderr,
	     "Usage: %s <sysnum> <topicnum> <trialnum> <permmatrix>\n", av[ 0 ] );
    exit( 1 );
  }

  sysnum = atol( *( av + 1 ) );
  topicnum = atol( *( av + 2 ) );
  trialnum = atol( *( av + 3 ) );

  if( ( f = fopen( *( av + 4 ), "r" ) ) == NULL ){
    fprintf( stderr, "Cannot open %s\n", *( av + 4 ) );
    exit( 1 );
  }

  trialcount = 1;
  linecount = 0;
  for( i = 0; i < SYSNUMMAX; i++ )
    sum[ i ] = 0.0;
  
  while( fgetls( &buf, f ) ){

    if( ( p = strtok( buf, SEP ) ) == NULL ){
      fprintf( stderr, "strtok failed\n" );
      return( -1 );
    }

    for( i = 0; i < sysnum; i++ ){

      if( ( p = strtok( NULL, SEP ) ) == NULL ){
	fprintf( stderr, "strtok failed\n" );
	return( -1 );
      }

      sum[ i ] += atof( p );

    }

    linecount++;

    if( linecount == topicnum ){ /* read one whole trial */

      max = sum[ 0 ]; min = sum[ 0 ];
      for( i = 1; i < sysnum; i++ ){
	if( sum[ i ] > max ){ max = sum[ i ]; }
	if( sum[ i ] < min ){ min = sum[ i ]; }
      }

      printf( "%ld %f %f %f\n", trialcount, max, min, (max - min)/topicnum );

      /* reset for next trial */
      
      trialcount++;
      linecount = 0;
      for( i = 0; i < SYSNUMMAX; i++ )
	sum[ i ] = 0.0;
    }

    free( buf );

  }

  fclose( f );


}/* of main */

/******************************************************************************
  printusage
******************************************************************************/
void printusage()
{

    fprintf( stderr, "Usage: cosine <vectorfile1> <vectorfile2>\n" );

}/* of printusage */
